# Ensure required services and COM objects are available
$netShareManager = New-Object -ComObject HNetCfg.HNetShare

$PublicAdapter = "Wi-Fi"
$PrivateAdapter = "Ethernet"

# Function to check Wi-Fi connection status
function Check-WiFiConnection {
    $wifiAdapter = Get-NetAdapter -Name $PublicAdapter -ErrorAction SilentlyContinue
    if ($wifiAdapter -and $wifiAdapter.Status -eq "Up") {
        return $true
    }
    return $false
}

# Wait for Wi-Fi connection to be established with timeout
while (-not (Check-WiFiConnection)) {
    Write-Host "Waiting for Wi-Fi connection..."
    Start-Sleep -Seconds 5
}

# Check if ICS is enabled on a specific connection
function IsICSEnabled {
    param(
        [string]$adapterName
    )
    $connection = $netShareManager.EnumEveryConnection | Where-Object {
        ($netShareManager.NetConnectionProps.Invoke($_)).Name -eq $adapterName
    }
    if (-not $connection) { return $false }
    
    $config = $netShareManager.INetSharingConfigurationForINetConnection.Invoke($connection)
    return $config.SharingEnabled
}

# Function to disable ICS on all connections
function Disable-ICS {
    $netShareManager.EnumEveryConnection | ForEach-Object {
        $config = $netShareManager.INetSharingConfigurationForINetConnection.Invoke($_)
        if ($config.SharingEnabled) {
            $config.DisableSharing()
            $connectionName = ($netShareManager.NetConnectionProps.Invoke($_)).Name
            Write-Host "ICS disabled on $connectionName."
        }
    }
}

# Function to enable ICS on a specific adapter
function Enable-ICS {
    param (
        [string]$publicAdapterName,
        [string]$privateAdapterName
    )

    $publicConnection = $netShareManager.EnumEveryConnection | Where-Object {
        ($netShareManager.NetConnectionProps.Invoke($_)).Name -eq $publicAdapterName
    }

    $privateConnection = $netShareManager.EnumEveryConnection | Where-Object {
        ($netShareManager.NetConnectionProps.Invoke($_)).Name -eq $privateAdapterName
    }

    if (-not $publicConnection -or -not $privateConnection) {
        Write-Error "Couldn't find the specified adapters."
        return
    }

    $publicConfig = $netShareManager.INetSharingConfigurationForINetConnection.Invoke($publicConnection)
    $privateConfig = $netShareManager.INetSharingConfigurationForINetConnection.Invoke($privateConnection)

    if (-not $publicConfig.SharingEnabled) {
        $publicConfig.EnableSharing(0)    # 0 = ICSSHARINGTYPE_PUBLIC
        Write-Host "ICS enabled on $publicAdapterName."
    }
    
    if (-not $privateConfig.SharingEnabled) {
        $privateConfig.EnableSharing(1)   # 1 = ICSSHARINGTYPE_PRIVATE
        Write-Host "ICS is shared to $privateAdapterName."
    }
}

# Main script execution

# Wait for Wi-Fi connection to be established
while (-not (Check-WiFiConnection)) {
    Write-Host "Waiting for Wi-Fi connection..."
    Start-Sleep -Seconds 5
}

if (IsICSEnabled -adapterName $PublicAdapter) {
    Disable-ICS
	Write-Host "Waiting 5 Seconds..."
    Start-Sleep -Seconds 5
}
else { Write-Host "ICS Already Disabled" }

# Enable ICS on 'Wi-Fi' and share to 'Ethernet'
Enable-ICS -publicAdapterName $PublicAdapter -privateAdapterName $PrivateAdapter

taskkill /IM "VirtualDesktop.Service.exe" /F
taskkill /IM "VirtualDesktop.Streamer.exe" /F

Start-Sleep -Seconds 3

Start-Process -FilePath "C:\Program Files\Virtual Desktop Streamer\VirtualDesktop.Streamer.exe"
# SIG # Begin signature block
# MIIPVgYJKoZIhvcNAQcCoIIPRzCCD0MCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUFJ6tvgWtDfQVJouvKej5CIWD
# YAagggyeMIIF2jCCBMKgAwIBAgIQB2JVTg9HNQsf7Wqb+UyhcDANBgkqhkiG9w0B
# AQsFADBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdpQ2VydCBFViBDb2Rl
# IFNpZ25pbmcgQ0EgKFNIQTIpMB4XDTIxMDQwNTAwMDAwMFoXDTI0MDUwODIzNTk1
# OVowgecxHTAbBgNVBA8MFFByaXZhdGUgT3JnYW5pemF0aW9uMRMwEQYLKwYBBAGC
# NzwCAQMTAkNBMSEwHwYLKwYBBAGCNzwCAQITEEJyaXRpc2ggQ29sdW1iaWExEjAQ
# BgNVBAUTCUJDMTEzMDk2MTELMAkGA1UEBhMCQ0ExGTAXBgNVBAgTEEJyaXRpc2gg
# Q29sdW1iaWExEjAQBgNVBAcTCVZhbmNvdXZlcjEeMBwGA1UEChMVVmlydHVhbCBE
# ZXNrdG9wLCBJbmMuMR4wHAYDVQQDExVWaXJ0dWFsIERlc2t0b3AsIEluYy4wggEi
# MA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDFHDvm9LVVdXBYVnKbtqiSBUux
# QDYMjb5Sf/2KaT60+iiGi/8//B6vsRfu7aMmyM3gSOLIEykGkhye6Lm8+owHkyeI
# NK+TLpZIPfUfSn9qifU3z+g/VARm6SkvIKXzfTmjvQZhiOvKCKuAlak0JDR98Jxd
# FW//OeLUxnE3/bTpoPRc5OuJA1EzA1qSKoxGgeFItVsaSFbKOPkf9QaAgQFq6jKV
# jxO95Rn3ELg5IQWwRm9nkyr9O1e/ygWNhAKTK14RNpfgY0UvAZlYPwNFmJ7UdEdm
# djpPJZUH/35V9/apfFWQrT2zvmMkmtCqsQrkK1P4DjOjHTZ7wxb1FOqFtOrpAgMB
# AAGjggH6MIIB9jAfBgNVHSMEGDAWgBSP6H7wbTJqAAUjx3CXajqQ/2vq1DAdBgNV
# HQ4EFgQUxY9x/eN3CNHYFzr0RKkKCTMHOcowOAYDVR0RBDEwL6AtBggrBgEFBQcI
# A6AhMB8MHUNBLUJSSVRJU0ggQ09MVU1CSUEtQkMxMTMwOTYxMA4GA1UdDwEB/wQE
# AwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzB7BgNVHR8EdDByMDegNaAzhjFodHRw
# Oi8vY3JsMy5kaWdpY2VydC5jb20vRVZDb2RlU2lnbmluZ1NIQTItZzEuY3JsMDeg
# NaAzhjFodHRwOi8vY3JsNC5kaWdpY2VydC5jb20vRVZDb2RlU2lnbmluZ1NIQTIt
# ZzEuY3JsMEoGA1UdIARDMEEwNgYJYIZIAYb9bAMCMCkwJwYIKwYBBQUHAgEWG2h0
# dHA6Ly93d3cuZGlnaWNlcnQuY29tL0NQUzAHBgVngQwBAzB+BggrBgEFBQcBAQRy
# MHAwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBIBggrBgEF
# BQcwAoY8aHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0RVZDb2Rl
# U2lnbmluZ0NBLVNIQTIuY3J0MAwGA1UdEwEB/wQCMAAwDQYJKoZIhvcNAQELBQAD
# ggEBAIeGQUW8PIl/hsdXOtOENP2L8OGIPmVzEbe98Z4T3KXw5yQgggu8tbyts5Vj
# 6ZFAnsQyAHWqq7z62OHv49wi9yyRIHymHOIiqoxqnm6BQga51MpDi9P4IrGRyqAB
# XJdOpAWprBGXigBi6trE3QSQe193NyRKxQm7T65CKbYJLkAYU/GynnIhxLKD56Zm
# u4Lqrm0ifQPXLG1wW2aNniR6H9LBGY4GEWgRD0UGEP32F7xVqfF896JgrsNP8UjB
# umEbeUtzf4PtnJWJfEZGMPgCJ4exJVUUGiaTvjt3e9PVewEdb9ShNAU/uU8GZecX
# dZU63EnR9XdpQp11pgrrK5nfrEMwgga8MIIFpKADAgECAhAD8bThXzqC8RSWeLPX
# 2EdcMA0GCSqGSIb3DQEBCwUAMGwxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxEaWdp
# Q2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xKzApBgNVBAMTIkRp
# Z2lDZXJ0IEhpZ2ggQXNzdXJhbmNlIEVWIFJvb3QgQ0EwHhcNMTIwNDE4MTIwMDAw
# WhcNMjcwNDE4MTIwMDAwWjBsMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNl
# cnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSswKQYDVQQDEyJEaWdp
# Q2VydCBFViBDb2RlIFNpZ25pbmcgQ0EgKFNIQTIpMIIBIjANBgkqhkiG9w0BAQEF
# AAOCAQ8AMIIBCgKCAQEAp1P6D7K1E/Fkz4SA/K6ANdG218ejLKwaLKzxhKw6NRI6
# kpG6V+TEyfMvqEg8t9Zu3JciulF5Ya9DLw23m7RJMa5EWD6koZanh08jfsNsZSSQ
# VT6hyiN8xULpxHpiRZt93mN0y55jJfiEmpqtRU+ufR/IE8t1m8nh4Yr4CwyY9Mo+
# 0EWqeh6lWJM2NL4rLisxWGa0MhCfnfBSoe/oPtN28kBa3PpqPRtLrXawjFzuNrqD
# 6jCoTN7xCypYQYiuAImrA9EWgiAiduteVDgSYuHScCTb7R9w0mQJgC3itp3OH/K7
# IfNs29izGXuKUJ/v7DYKXJq3StMIoDl5/d2/PToJJQIDAQABo4IDWDCCA1QwEgYD
# VR0TAQH/BAgwBgEB/wIBADAOBgNVHQ8BAf8EBAMCAYYwEwYDVR0lBAwwCgYIKwYB
# BQUHAwMwfwYIKwYBBQUHAQEEczBxMCQGCCsGAQUFBzABhhhodHRwOi8vb2NzcC5k
# aWdpY2VydC5jb20wSQYIKwYBBQUHMAKGPWh0dHA6Ly9jYWNlcnRzLmRpZ2ljZXJ0
# LmNvbS9EaWdpQ2VydEhpZ2hBc3N1cmFuY2VFVlJvb3RDQS5jcnQwgY8GA1UdHwSB
# hzCBhDBAoD6gPIY6aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0SGln
# aEFzc3VyYW5jZUVWUm9vdENBLmNybDBAoD6gPIY6aHR0cDovL2NybDQuZGlnaWNl
# cnQuY29tL0RpZ2lDZXJ0SGlnaEFzc3VyYW5jZUVWUm9vdENBLmNybDCCAcQGA1Ud
# IASCAbswggG3MIIBswYJYIZIAYb9bAMCMIIBpDA6BggrBgEFBQcCARYuaHR0cDov
# L3d3dy5kaWdpY2VydC5jb20vc3NsLWNwcy1yZXBvc2l0b3J5Lmh0bTCCAWQGCCsG
# AQUFBwICMIIBVh6CAVIAQQBuAHkAIAB1AHMAZQAgAG8AZgAgAHQAaABpAHMAIABD
# AGUAcgB0AGkAZgBpAGMAYQB0AGUAIABjAG8AbgBzAHQAaQB0AHUAdABlAHMAIABh
# AGMAYwBlAHAAdABhAG4AYwBlACAAbwBmACAAdABoAGUAIABEAGkAZwBpAEMAZQBy
# AHQAIABDAFAALwBDAFAAUwAgAGEAbgBkACAAdABoAGUAIABSAGUAbAB5AGkAbgBn
# ACAAUABhAHIAdAB5ACAAQQBnAHIAZQBlAG0AZQBuAHQAIAB3AGgAaQBjAGgAIABs
# AGkAbQBpAHQAIABsAGkAYQBiAGkAbABpAHQAeQAgAGEAbgBkACAAYQByAGUAIABp
# AG4AYwBvAHIAcABvAHIAYQB0AGUAZAAgAGgAZQByAGUAaQBuACAAYgB5ACAAcgBl
# AGYAZQByAGUAbgBjAGUALjAdBgNVHQ4EFgQUj+h+8G0yagAFI8dwl2o6kP9r6tQw
# HwYDVR0jBBgwFoAUsT7DaQP4v0cB1JgmGggC72NkK8MwDQYJKoZIhvcNAQELBQAD
# ggEBABkzSgyBMzfbrTbJ5Mk6u7UbLnqi4vRDQheev06hTeGx2+mB3Z8B8uSI1en+
# Cf0hwexdgNLw1sFDwv53K9v515EzzmzVshk75i7WyZNPiECOzeH1fvEPxllWcujr
# akG9HNVG1XxJymY4FcG/4JFwd4fcyY0xyQwpojPtjeKHzYmNPxv/1eAal4t82m37
# qMayOmZrewGzzdimNOwSAauVWKXEU1eoYObnAhKguSNkok27fIElZCG+z+5CGEOX
# u6U3Bq9N/yalTWFL7EZBuGXOuHmeCJYLgYyKO4/HmYyjKm6YbV5hxpa3irlhLZO4
# 6w4EQ9f1/qbwYtSZaqXBwfBklIAxggIiMIICHgIBATCBgDBsMQswCQYDVQQGEwJV
# UzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQu
# Y29tMSswKQYDVQQDEyJEaWdpQ2VydCBFViBDb2RlIFNpZ25pbmcgQ0EgKFNIQTIp
# AhAHYlVOD0c1Cx/tapv5TKFwMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3AgEMMQow
# CKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcC
# AQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBQYLnnLNEJpx1/Wb6WQ
# 2HDSNEEghTANBgkqhkiG9w0BAQEFAASCAQBCaHLumSEFd2AzhPZLI9qMwTvNrPZf
# 6OC5+bIOABWlYoY0HwcEEcmRcb9/avH3FZFnwcwn5Y17u9Urs4A3sCvrtqGmNwpV
# ozPKOPztaK+X8Xn2n3BMwkuBjbIC4mrmePhjTZVtGnnYdi6wDyramcTQzfoKaEjp
# 9O7WYQwWDF9veNJ8XQzf6d1QnhVyPbplw+8lAb2fSKUa5GJ+gEJdzSfh9fSc61Gk
# lSF5X+zKetCclHFzoCGp+DVqJmIl/MULu6ZNDrKTqAerFJq6zLOFSuhhXD9k0J16
# y3JjtYbyGr8u5avm3VaFJhN6rDRjdhlz2fFqrB0dr1wze46FeIOCFM8x
# SIG # End signature block
